@@ -24,6 +24,7 @@
  */
 
 #include "xslt_i18n.h"
+#include "gsad_base.h"
 #include <assert.h>
 #include <dirent.h>
 #include <errno.h>
@@ -464,6 +465,15 @@ xslt_ext_strformat (xmlXPathParserContextPtr ctxt,
     }
 
   result_obj = xmlXPathNewString ((xmlChar*) result_str->str);
+
+  {
+    guint index = format_args->len;
+    while (index--)
+      g_free (g_array_index (format_args, gchar*, index));
+    g_array_free (format_args, TRUE);
+  }
+  g_free (format_string);
+  g_string_free (number_str, TRUE);
   g_string_free (result_str, TRUE);
   valuePush (ctxt, result_obj);
 }
@@ -528,7 +538,10 @@ register_i18n_ext_module ()
                         init_i18n_module,
                         shutdown_i18n_module);
 
-  if (bindtextdomain (GSA_XSL_TEXTDOMAIN, GSA_LOCALE_DIR) == NULL)
+  if (bindtextdomain (GSA_XSL_TEXTDOMAIN,
+                      get_chroot_state () ? GSA_CHROOT_LOCALE_DIR
+                                          : GSA_LOCALE_DIR)
+      == NULL)
     {
       g_critical ("%s: Failed to bind text domain for gettext", __FUNCTION__);
       abort ();
@@ -567,6 +580,7 @@ int
 init_language_lists ()
 {
   FILE *lang_names_file;
+  const char *locale_dir_name;
   DIR *locale_dir;
   struct dirent *entry;
 
@@ -626,6 +640,7 @@ init_language_lists ()
                 g_hash_table_insert (native_language_names,
                                      g_strdup (code),
                                      g_strdup (native_name));
+              g_strfreev (columns);
             }
           g_free (line);
           line = NULL;
@@ -639,7 +654,9 @@ init_language_lists ()
     }
 
   /* Get installed translations */
-  locale_dir = opendir (GSA_LOCALE_DIR);
+  locale_dir_name = get_chroot_state () ? GSA_CHROOT_LOCALE_DIR 
+                                        : GSA_LOCALE_DIR;
+  locale_dir = opendir (locale_dir_name);
 
   if (locale_dir == NULL)
     {
@@ -658,7 +675,7 @@ init_language_lists ()
         {
           FILE *mo_file;
           gchar *lang_mo_path;
-          lang_mo_path = g_build_filename (GSA_LOCALE_DIR,
+          lang_mo_path = g_build_filename (locale_dir_name,
                                            entry->d_name,
                                            "LC_MESSAGES",
                                            GSA_XSL_TEXTDOMAIN ".mo",
@@ -694,7 +711,7 @@ init_language_lists ()
 
 /**
  * @brief Write the list of installed languages to a buffer as XML.
- * 
+ *
  * @param[in] buffer  A GString buffer to write to.
  */
 void
@@ -744,7 +761,9 @@ buffer_languages_xml (GString *buffer)
  *  in the LANGUAGE environment variable.
  *
  * @param[in]   accept_language  HTTP Accept-Language header text.
- * @return      Newly allocated strig of language codes as used by gettext.
+ * @return      Newly allocated string of language codes as used by gettext. If
+ *              accept_language is NULL or it doesn't contain a language
+ *              DEFAULT_GSAD_LANGUAGE is returned.
  */
 gchar *
 accept_language_to_env_fmt (const char* accept_language)
@@ -775,4 +794,4 @@ accept_language_to_env_fmt (const char* accept_language)
   g_strfreev (prefs);
 
   return language;
-}
\ No newline at end of file
+}
